import urllib.parse as parse
import html.parser as parser


class BayParser(parser.HTMLParser):
    title = ''
    q = ''
    state = 'looking'
    results = []

    def handle_starttag(self, tag, attrs):
        if tag == 'title':
            self.state = 'title'
        if tag == 'magnet' and self.state == 'matched':
            self.state = 'magnet'

    def handle_data(self, data):
        if self.state == 'title':
            if data.lower().find(self.q) != -1:
                self.title = data
                self.state = 'matched'
            else:
                self.state = 'looking'
        if self.state == 'magnet':
            self.results.append([
                'magnet:?xt=urn:btih:' +
                parse.quote(data) +
                '&dn=' +
                parse.quote(self.title), '?', '?'])
            self.state = 'looking'


def search(db, terms):
    f = open(db)
    xml = f.readlines()
    f.close()
    parser = BayParser()
    parser.q = (' '.join(terms)).lower()
    parser.feed(''.join(xml))
    return parser.results



#!/usr/bin/env python3
import unittest
import pirate.local
import os

from tests import util


class TestLocal(unittest.TestCase):

    def test_rich_xml(self):
        path = util.data_path('rich.xml')
        expected = [['magnet:?xt=urn:btih:b03c8641415d3a0fc7077f5bf567634442989a74&dn=High.Chaparall.S02E02.PDTV.XViD.SWEDiSH-HuBBaTiX', '?', '?']]
        actual = pirate.local.search(path, ('High',))
        self.assertEqual(actual, expected)

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python3
import socket
import unittest
import subprocess
from argparse import Namespace
from unittest import mock
from unittest.mock import patch, call, MagicMock

import pirate.pirate
import pirate.data
from pirate.print import Printer


class TestPirate(unittest.TestCase):

    def test_parse_cmd(self):
        tests = [
            [['abc', ''], ['abc']],
            [['abc %s', 'url'], ['abc', 'url']],
            [['abc "%s"', 'url'], ['abc', 'url']],
            [["abc \'%s\'", 'url'], ['abc', 'url']],
            [['abc bash -c "\'%s\'"', 'url'], ['abc', 'bash', '-c', "'url'"]],
            [['abc %s %s', 'url'], ['abc', 'url', 'url']],
        ]
        for test in tests:
            self.assertEqual(pirate.pirate.parse_cmd(*test[0]), test[1])

    @patch('subprocess.call')
    def test_main(self, mock_call):
        result = {
            'magnet': 'dn=derp',
            'seeds': '1',
            'leechers': '1',
            'size': ('1', 'mb'),
            'uploaded': '1',
        }
        with patch('pirate.torrent.remote', return_value=[result]) as mock_remote:
            config = pirate.pirate.parse_config_file('')
            args = pirate.pirate.combine_configs(config, pirate.pirate.parse_args(['-0', 'term', '-C', 'blah %s']))
            pirate.pirate.pirate_main(args)
            mock_call.assert_called_once_with(['blah', 'dn=derp'])

    @patch('pirate.pirate.builtins.input', return_value='0')
    @patch('subprocess.call')
    def test_main_choice(self, mock_call, mock_input):
        result = {
            'magnet': 'dn=derp',
            'seeds': '1',
            'leechers': '1',
            'size': ('1', 'mb'),
            'uploaded': '1',
        }
        with patch('pirate.torrent.remote', return_value=[result]) as mock_remote:
            config = pirate.pirate.parse_config_file('')
            args = pirate.pirate.combine_configs(config, pirate.pirate.parse_args(['term', '-C', 'blah %s']))
            pirate.pirate.pirate_main(args)
            mock_call.assert_called_once_with(['blah', 'dn=derp'])

    def test_parse_torrent_command(self):
        tests = [
            [['h'], ('h', [])],
            [['q'], ('q', [])],
            [['d1'], ('d', [1])],
            [['f1'], ('f', [1])],
            [['p1'], ('p', [1])],
            [['t1'], ('t', [1])],
            [['m1'], ('m', [1])],
            [['d 23'], ('d', [23])],
            [['d 23,1'], ('d', [23, 1])],
            [['d 23, 1'], ('d', [23, 1])],
            [['1d'], ('d', [1])],
            [['1 ... d'], ('d', [1])],
            [['1-3 d'], ('d', [1,2,3])],
            [['1-3'], (None, [1,2,3])],
        ]
        for test in tests:
            self.assertEqual(pirate.pirate.parse_torrent_command(*test[0]), test[1])

    def test_parse_config_file(self):
        types = {
            'Save': {
                'magnets': bool,
                'Magnets': bool,
                'torrents': bool,
                'directory': str,
            },
            'LocalDB': {
                'enabled': bool,
                'path': str,
            }
        }
        config1 = """
        [Save]
        magnets=False
        directory=dir
        [LocalDB]
        enabled=true
        path=abc
        """
        config2 = """
        [Save]
        Magnets=True
        """
        tests = [
            (config1, {'Save': {'magnets': False}}),
            (config1, {'Save': {'torrents': False}}),
            (config1, {'Save': {'directory': 'dir'}}),
            (config1, {'LocalDB': {'enabled': True}}),
            (config1, {'LocalDB': {'path': 'abc'}}),
            (config2, {'Save': {'magnets': True}}),
        ]
        for test in tests:
            config = pirate.pirate.parse_config_file(test[0])
            for section in test[1].keys():
                for name in test[1][section].keys():
                    if types[section][name] == bool:
                        lhs = config.getboolean(section, name)
                    else:
                        lhs = config.get(section, name)
                    rhs = test[1][section][name]
                    self.assertEqual(lhs, rhs)

    def test_parse_args(self):
        tests = [
            ('', ['-b'], {'action': 'browse'}),
            ('', [], {'action': 'top'}),
            ('', ['-R'], {'action': 'recent'}),
            ('', ['-l'], {'action': 'list_categories'}),
            ('', ['--list_sorts'], {'action': 'list_sorts'}),
            ('', ['term'], {'action': 'search', 'source': 'tpb'}),
            ('', ['-L', 'filename', 'term'], {'action': 'search', 'source': 'local_tpb', 'database': 'filename'}),
            ('', ['term', '-S', 'dir'], {'action': 'search', 'save_directory': 'dir'}),
            ('', ['-P', '1337'], {'transmission_command': ['transmission-remote', '1337']}),
            ('', ['term'], {'output': 'browser_open'}),
            ('', ['term', '-t'], {'output': 'transmission'}),
            ('', ['term', '--save-magnets'], {'output': 'save_magnet_files'}),
            ('', ['term', '--save-torrents'], {'output': 'save_torrent_files'}),
            ('', ['term', '-C', 'command'], {'output': 'open_command', 'open_command': 'command'}),
            ('', ['internets'], {'action': 'search', 'search': ['internets']}),
            ('', ['internets lol', 'lel'], {'action': 'search', 'search': ['internets lol', 'lel']}),
        ]
        for test in tests:
            args = pirate.pirate.parse_args(test[1])
            config = pirate.pirate.parse_config_file(test[0])
            args = pirate.pirate.combine_configs(config, args)
            for option in test[2].keys():
                value = getattr(args, option)
                self.assertEqual(test[2][option], value)

    def test_search_mirrors(self):
        args = Namespace(pages=1, category=100, sort=10,
                         action='browse', search=[],
                         mirror=[pirate.data.default_mirror])
        class MockResponse():
            readlines = mock.MagicMock(return_value=[x.encode('utf-8') for x in ['', '', '', 'https://example.com']])
            info = mock.MagicMock()
            getcode = mock.MagicMock(return_value=200)
        response_obj = MockResponse()
        printer = MagicMock(Printer)
        with patch('urllib.request.urlopen', return_value=response_obj) as urlopen:
            with patch('pirate.torrent.remote', return_value=[]) as remote:
                results, mirror = pirate.pirate.search_mirrors(printer, args)
                self.assertEqual(results, [])
                self.assertEqual(mirror, pirate.data.default_mirror)
                remote.assert_called_once_with(printer=printer, pages=1, category=100, sort=10, mode='browse', terms=[], mirror=pirate.data.default_mirror)
            with patch('pirate.torrent.remote', side_effect=[socket.timeout, []]) as remote:
                results, mirror = pirate.pirate.search_mirrors(printer, args)
                self.assertEqual(results, [])
                self.assertEqual(mirror, 'https://example.com')
                remote.assert_has_calls([
                    call(printer=printer, pages=1, category=100, sort=10, mode='browse', terms=[], mirror=pirate.data.default_mirror),
                    call(printer=printer, pages=1, category=100, sort=10, mode='browse', terms=[], mirror='https://example.com')
                ])

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python3
from setuptools import setup, find_packages
from distutils.version import LooseVersion
import sys
import pirate.data

if LooseVersion(sys.version) < LooseVersion("3.4.0"):
    print("pirate-get requires at least python 3.4.0."
          " Your version is %s." % sys.version.split()[0])
    sys.exit(1)

if __name__ == '__main__':
    setup(name='pirate-get',
        version=pirate.data.version,
        description='A command line interface for The Pirate Bay',
        url='https://github.com/vikstrous/pirate-get',
        author='vikstrous',
        author_email='me@viktorstanchev.com',
        license='AGPL',
        packages=find_packages(),
        package_data={'': ["data/*.json"]},
        entry_points={
            'console_scripts': ['pirate-get = pirate.pirate:main']
        },
        install_requires=['colorama>=0.3.3',
                          'beautifulsoup4>=4.4.1',
                          'veryprettytable>=0.8.1'],
        keywords=['torrent', 'magnet', 'download', 'tpb', 'client'],
        classifiers=[
            'Topic :: Utilities',
            'Topic :: Terminals',
            'Topic :: System :: Networking',
            'Programming Language :: Python :: 3 :: Only',
            'Programming Language :: Python :: 3.4',
            'License :: OSI Approved :: GNU General Public License (GPL)',
        ],
        test_suite='tests')



#!/usr/bin/env python3
import sys
import os.path
import unittest
import importlib
from unittest import mock
from unittest.mock import patch, call, MagicMock

sys.path.append(os.path.join(os.path.dirname(__file__), '..'))

import setup

class TestPirate(unittest.TestCase):

    @patch('sys.exit')
    def test_unsupported(self, mock_exit):
        sys.version = '3.2.1 (default, dec  7 2015, 12:58:09) \n[gcc 5.2.0]'
        importlib.reload(setup)
        mock_exit.assert_called_once_with(1)

    @patch('sys.exit')
    def test_unsupported2(self, mock_exit):
        sys.version = '2.5.1 (default, dec  7 2015, 12:58:09) \n[gcc 5.2.0]'
        importlib.reload(setup)
        mock_exit.assert_called_once_with(1)


    @patch('sys.exit')
    def test_supported(self, mock_exit):
        sys.version = '3.5.1 (default, dec  7 2015, 12:58:09) \n[gcc 5.2.0]'
        importlib.reload(setup)
        mock_exit.assert_not_called()

    @patch('sys.exit')
    def test_supported_exact(self, mock_exit):
        sys.version = '3.4.0 (default, dec  7 2015, 12:58:09) \n[gcc 5.2.0]'
        importlib.reload(setup)
        mock_exit.assert_not_called()

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python3
import unittest
from unittest.mock import patch, call, MagicMock

from pirate.print import Printer


class TestPrint(unittest.TestCase):

    def test_print_results_remote(self):
        class MockTable:
            add_row = MagicMock()
            align = {}
        mock = MockTable()
        printer = Printer(False)
        printer.print = MagicMock()
        with patch('veryprettytable.VeryPrettyTable', return_value=mock) as prettytable:
            results = [{
                'magnet': 'dn=name',
                'seeds': 1,
                'leechers': 2,
                'size': ['3','MiB'],
                'uploaded': 'never'
            }]
            printer.search_results(results)
            prettytable.assert_called_once_with(['LINK', 'SEED', 'LEECH', 'RATIO', 'SIZE', '', 'UPLOAD', 'NAME'])
            mock.add_row.assert_has_calls([call([0, 1, 2, '0.5', '3.0', 'MiB', 'never', 'name'])])

    def test_print_results_local(self):
        class MockTable:
            add_row = MagicMock()
            align = {}
        mock = MockTable()
        printer = Printer(False)
        printer.print = MagicMock()
        with patch('veryprettytable.VeryPrettyTable', return_value=mock) as prettytable:
            results = [{
                'magnet': 'dn=name',
                'Name': 'name',
            },{
                'magnet': 'dn=name2',
                'Name': 'name2',
            }]
            printer.search_results(results, local=True)
            prettytable.assert_called_once_with(['LINK', 'NAME'])
            mock.add_row.assert_has_calls([call([0, 'name']), call([1, 'name2'])])

    def test_print_color(self):
        printer = Printer(False)
        with patch('pirate.print.builtins.print') as mock_print:
            printer.print('abc', color='zebra_1')
            mock_print.assert_called_once_with('abc')
        printer = Printer(True)
        with patch('pirate.print.builtins.print') as mock_print:
            printer.print('abc', color='zebra_1')
            mock_print.assert_called_once_with('\x1b[34mabc', '\x1b[0m')

    def test_print_results_local(self):
        class MockTable:
            add_row = MagicMock()
            align = {}
        mock = MockTable()
        printer = Printer(True)
        printer.print = MagicMock()
        with patch('veryprettytable.VeryPrettyTable', return_value=mock) as prettytable:
            results = [{
                'magnet': 'dn=name',
                'Name': 'name',
            },{
                'magnet': 'dn=name2',
                'Name': 'name2',
            }]
            printer.search_results(results, local=True)
            prettytable.assert_called_once_with(['LINK', 'NAME'])
            mock.add_row.assert_has_calls([call([0, 'name']), call([1, 'name2'], fore_color='blue')])

    def test_print_descriptions(self):
        printer = Printer(False)
        printer.print = MagicMock()
        class MockRequest():
            add_header = MagicMock()
        request_obj = MockRequest()
        class MockResponse():
            read = MagicMock(return_value='<html><div class="nfo"><pre>stuff <a href="href">link</a></pre></div></html>'.encode('utf8'))
            info = MagicMock()
        response_obj = MockResponse()
        with patch('urllib.request.Request', return_value=request_obj) as request:
            with patch('urllib.request.urlopen', return_value=response_obj) as urlopen:
                printer.descriptions([0], [{'id': '1', 'magnet': 'dn=name'}], 'example.com')
                printer.print.assert_has_calls([call('Description for "name":', color='zebra_1'),call('stuff [link](href)', color='zebra_0')])

    def test_print_file_lists(self):
        printer = Printer(False)
        printer.print = MagicMock()
        class MockRequest():
            add_header = MagicMock()
        request_obj = MockRequest()
        class MockResponse():
            read = MagicMock(return_value='<html><tr><td align="left">1.</td><td align="right">filename</tr></html>'.encode('utf8'))
            info = MagicMock()
        response_obj = MockResponse()
        with patch('urllib.request.Request', return_value=request_obj) as request:
            with patch('urllib.request.urlopen', return_value=response_obj) as urlopen:
                printer.file_lists([0], [{'id': '1', 'magnet': 'dn=name'}], 'example.com')
                printer.print.assert_has_calls([call('Files in "name":', color='zebra_1'),call('         1.  filename', color='zebra_0')])

if __name__ == '__main__':
    unittest.main()



import re
import sys
import gzip
import urllib.request as request
import urllib.parse as parse
import urllib.error
import os.path

from bs4 import BeautifulSoup

import pirate.data

from io import BytesIO


parser_regex = r'"(magnet\:\?xt=[^"]*)|<td align="right">([^<]+)</td>'


def parse_category(printer, category):
    try:
        category = int(category)
    except ValueError:
        pass
    if category in pirate.data.categories.values():
        return category
    elif category in pirate.data.categories.keys():
        return pirate.data.categories[category]
    else:
        printer.print('Invalid category ignored', color='WARN')
        return 0


def parse_sort(printer, sort):
    try:
        sort = int(sort)
    except ValueError:
        pass
    if sort in pirate.data.sorts.values():
        return sort
    elif sort in pirate.data.sorts.keys():
        return pirate.data.sorts[sort]
    else:
        printer.print('Invalid sort ignored', color='WARN')
        return 99


# TODO:
# * warn users when using a sort in a mode that doesn't accept sorts
# * warn users when using search terms in a mode
#   that doesn't accept search terms
# * same with page parameter for top and top48h
# * warn the user if trying to use a minor category with top48h
def build_request_path(page, category, sort, mode, terms):
    if mode == 'browse':
        if(category == 0):
            category = 100
        return '/browse/{}/{}/{}'.format(category, page, sort)
    elif mode == 'recent':
        # This is not a typo. There is no / between 48h and the category.
        path = '/top/48h'
        # only major categories can be used with this mode
        if(category == 0):
            return path + 'all'
        else:
            return path + str(category)
    elif mode == 'top':
        path = '/top/'
        if(category == 0):
            return path + 'all'
        else:
            return path + str(category)
    elif mode == 'search':
        query = urllib.parse.quote_plus(' '.join(terms))
        return '/search/{}/{}/{}/{}'.format(query, page, sort, category)
    else:
        raise Exception('Unknown mode.')


# this returns a list of dictionaries
def parse_page(html):
    soup = BeautifulSoup(html, 'html.parser')
    table = soup.find('table', id='searchResult')

    results = []
    no_results = re.search(r'No hits\. Try adding an asterisk in '
                           r'you search phrase\.', html)

    # check for a blocked mirror
    if not table and not no_results:
        # Contradiction - we found no results,
        # but the page didn't say there were no results.
        # The page is probably not actually the pirate bay,
        # so let's try another mirror
        raise IOError('Blocked mirror detected.')

    if no_results:
        return results

    # parse the rows one by one (skipping headings)
    for row in table('tr')[1:]:
        # grab info about the row
        id_ = row.find('a', class_='detLink')['href'].split('/')[2]
        seeds, leechers = [i.text for i in row('td')[-2:]]
        magnet = row.find(lambda tag:
                          tag.name == 'a' and
                          tag['href'].startswith('magnet'))['href']

        # parse descriptions separately
        description = row.find('font', class_='detDesc').text
        size = re.findall(r'(?<=Size )[0-9.]+\s[KMGT]*[i ]*B',
                          description)[0].split()
        uploaded = re.findall(r'(?<=Uploaded ).+(?=\, Size)',
                              description)[0]

        results.append({
            'magnet': magnet,
            'seeds': seeds,
            'leechers': leechers,
            'size': size,
            'uploaded': uploaded,
            'id': id_
        })

    return results


def remote(printer, pages, category, sort, mode, terms, mirror):
    res_l = []

    if pages < 1:
        raise ValueError('Please provide an integer greater than 0 '
                         'for the number of pages to fetch.')

    # Catch the Ctrl-C exception and exit cleanly
    try:
        for page in range(pages):
            path = build_request_path(page, category, sort, mode, terms)

            req = request.Request(mirror + path,
                                  headers=pirate.data.default_headers)
            req.add_header('Accept-encoding', 'gzip')
            f = request.urlopen(req, timeout=pirate.data.default_timeout)
            if f.info().get('Content-Encoding') == 'gzip':
                f = gzip.GzipFile(fileobj=BytesIO(f.read()))
            res = f.read().decode('utf-8')

            res_l += parse_page(res)

    except KeyboardInterrupt:
        printer.print('\nCancelled.')
        sys.exit(0)

    return res_l


def get_torrent(info_hash):
    url = 'http://itorrents.org/torrent/{:X}.torrent'
    req = request.Request(url.format(info_hash),
                          headers=pirate.data.default_headers)
    req.add_header('Accept-encoding', 'gzip')

    torrent = request.urlopen(req, timeout=pirate.data.default_timeout)
    if torrent.info().get('Content-Encoding') == 'gzip':
        torrent = gzip.GzipFile(fileobj=BytesIO(torrent.read()))

    return torrent.read()


def save_torrents(printer, chosen_links, results, folder):
    for link in chosen_links:
        magnet = results[link]['magnet']
        name = re.search(r'dn=([^\&]*)', magnet)
        torrent_name = parse.unquote(name.group(1)).replace('+', ' ')
        info_hash = int(re.search(r'btih:([a-f0-9]{40})', magnet).group(1), 16)
        torrent_name = torrent_name.replace('/', '_').replace('\\', '_')
        file = os.path.join(folder, torrent_name + '.torrent')

        try:
            torrent = get_torrent(info_hash)
        except urllib.error.HTTPError:
            printer.print('There is no cached file for this torrent :(',
                          color='ERROR')
        else:
            open(file, 'wb').write(torrent)
            printer.print('Saved {:X} in {}'.format(info_hash, file))


def save_magnets(printer, chosen_links, results, folder):
    for link in chosen_links:
        magnet = results[link]['magnet']
        name = re.search(r'dn=([^\&]*)', magnet)
        torrent_name = parse.unquote(name.group(1)).replace('+', ' ')
        info_hash = int(re.search(r'btih:([a-f0-9]{40})', magnet).group(1), 16)
        torrent_name = torrent_name.replace('/', '_').replace('\\', '_')
        file = os.path.join(folder,  torrent_name + '.magnet')

        printer.print('Saved {:X} in {}'.format(info_hash, file))
        with open(file, 'w') as f:
            f.write(magnet + '\n')
